///////////////////////////////////////////////////////////////////////////////
// File:	wx_rbut.cc
// Purpose:	Panel item radioButton implementation (Macintosh version)
// Author:	Bill Hale
// Created:	1994
// Updated:	
// Copyright:  (c) 1993-94, AIAI, University of Edinburgh. All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////

#include "wx_rbut.h"
#include "wx_utils.h"
#include "wx_mac_utils.h"
#include "wxMacDC.h"
#include "wx_stdev.h"
#include "wx_panel.h"
#include "wx_area.h"
#ifndef WX_CARBON
# include <QuickDraw.h>
#endif

#define IR_CIRCLE_SIZE 12
#define IR_X_SPACE 3
#define IR_Y_SPACE 2
#define IR_MIN_HEIGHT (IR_CIRCLE_SIZE + 2 * IR_Y_SPACE)
#define IR_ON_INSET 3

//=============================================================================
// Public constructors
//=============================================================================

//-----------------------------------------------------------------------------
wxRadioButton::wxRadioButton // Constructor (given parentPanel, label)
(
 wxPanel*	parentPanel,
 wxFunction	function,
 char*		label,
 int 		x,
 int			y,
 int			width,
 int			height,
 long		style,
 char*		windowName,
 WXTYPE		objectType
 ) :
 wxItem (parentPanel, x, y, width, height, style, windowName)
{
  Create(parentPanel, function, label, x, y, width, height, style, windowName, objectType);
}

void wxRadioButton::Create // Real constructor (given parentPanel, label)
(
 wxPanel*	parentPanel,
 wxFunction	function,
 char*		label,
 int 		x,
 int			y,
 int			width,
 int			height,
 long		style,
 char*		windowName,
 WXTYPE		objectType
 ) 
{
  OSErr err;
  CGrafPtr theMacGrafPort;
  Rect boundsRect = {0, 0, 0, 0};
  CFStringRef theMacLabel;
  SInt16 baselineOffset; // ignored

  buttonBitmap = NULL;
  Callback(function);
  
  font = buttonFont; // WCH: mac platform only

  // First, create the control with a bogus rectangle;
  SetCurrentMacDC();
  theMacGrafPort = cMacDC->macGrafPort();
  OffsetRect(&boundsRect,SetOriginX,SetOriginY);
  theMacLabel = CFStringCreateWithCString(NULL, label, kCFStringEncodingISOLatin1);
  
  err = CreateRadioButtonControl(GetWindowFromPort(theMacGrafPort), &boundsRect, theMacLabel,
				 0, FALSE, &cMacControl);

  CFRelease(theMacLabel);
  
  // Now, ignore the font data and let the control find the "best" size 
  err = ::GetBestControlRect(cMacControl,&boundsRect,&baselineOffset);
  cWindowWidth = boundsRect.right - boundsRect.left;
  cWindowHeight = boundsRect.bottom - boundsRect.top;
  ::SizeControl(cMacControl, boundsRect.right - boundsRect.left, boundsRect.bottom - boundsRect.top);

  ::EmbedControl(cMacControl, GetRootControl());

  {
    wxWindow *p;
    p = GetParent();
    if (p->IsHidden())
      DoShow(FALSE);
  }
  InitInternalGray();
}

//-----------------------------------------------------------------------------
wxRadioButton::wxRadioButton // Constructor (given parentPanel, bitmap)
(
 wxPanel*	parentPanel,
 wxFunction	function,
 wxBitmap*	bitmap,
 int 		x,
 int			y,
 int			width,
 int			height,
 long		style,
 char*		windowName,
 WXTYPE		objectType
 ) :
 wxItem (parentPanel, x, y, width, height, style, windowName)
{
  Rect bounds;

  if (bitmap->Ok() && (bitmap->selectedIntoDC >= 0)) {
    buttonBitmap = bitmap;
    buttonBitmap->selectedIntoDC++;
  } else {
    Create(parentPanel, function, "<bad-image>", x, y, width, height, style, windowName, objectType);
  }

  Callback(function);
  cMacControl = NULL;
  
  ::SetRect(&bounds, 0, 0, buttonBitmap->GetWidth(), buttonBitmap->GetHeight());
  cWindowHeight = bounds.bottom;
  cWindowWidth = bounds.right + IR_CIRCLE_SIZE + IR_X_SPACE;
  if (cWindowHeight < IR_MIN_HEIGHT)
    cWindowHeight = IR_MIN_HEIGHT;
  OffsetRect(&bounds,SetOriginX,SetOriginY);
  
  {
    wxWindow *p;
    p = GetParent();
    if (p->IsHidden())
      DoShow(FALSE);
  }
  
  {
    int vis;
    vis = SetCurrentMacDC();
    if (vis) {
      ::InvalWindowRect(GetWindowFromPort(cMacDC->macGrafPort()),&bounds);
    }
  }

  InitInternalGray();
}

//=============================================================================
// Public destructor
//=============================================================================

//-----------------------------------------------------------------------------
wxRadioButton::~wxRadioButton(void)
{
  if (cMacControl) { ::DisposeControl(cMacControl); }
  if (buttonBitmap)
    --buttonBitmap->selectedIntoDC;
}


//-----------------------------------------------------------------------------
void wxRadioButton::ChangeColour(void)
{
}

//-----------------------------------------------------------------------------
char* wxRadioButton::GetLabel()
{
  if (cMacControl) {
    Str255	pLabel;

    ::GetControlTitle(cMacControl, pLabel);
    ::CopyPascalStringToC(pLabel, wxBuffer);
    return copystring(wxBuffer);
  } else if (labelString)
    return labelString;
  else
    return NULL;
}

//-----------------------------------------------------------------------------
void wxRadioButton::SetLabel(char* label)
{
  if (label && !buttonBitmap) {
    if (cMacControl) {
      SetCurrentDC();
      {
	CFStringRef llabel;
	llabel = CFStringCreateWithCString(NULL, label, kCFStringEncodingISOLatin1);
	SetControlTitleWithCFString(cMacControl, llabel);
	CFRelease(llabel);
      }
    } else
      labelString = label;
  }
}

//-----------------------------------------------------------------------------
void wxRadioButton::SetLabel(wxBitmap* bitmap)
{
  if (buttonBitmap && bitmap->Ok() && (bitmap->selectedIntoDC >= 0)) {
    --buttonBitmap->selectedIntoDC;
    buttonBitmap = bitmap;
    buttonBitmap->selectedIntoDC++;
    Refresh();
  }
}

//-----------------------------------------------------------------------------
void wxRadioButton::SetValue(Bool val)
{
  if (cMacControl) {
    SetCurrentDC();
    ::SetControlValue(cMacControl, val ? 1 : 0);
  } else {
    bitmapState = !!val;
    if (!cHidden) {
      Paint();
      /* in case paint didn't take, because an update is already in progress: */
      Refresh();
    }
  }
}

//-----------------------------------------------------------------------------
Bool wxRadioButton::GetValue(void)
{
  if (cMacControl) {
    short value;
    value = ::GetControlValue(cMacControl);
    return (value != 0) ? TRUE : FALSE;
  } else
    return bitmapState;
}

//-----------------------------------------------------------------------------
void wxRadioButton::Paint(void)
{
  if (cHidden) return;
  SetCurrentDC();
  { 
    Rect r;

    SetRect(&r, 0, 0, cWindowWidth, cWindowHeight);

    OffsetRect(&r,SetOriginX,SetOriginY);
    ::EraseRect(&r);
    if (cMacControl) {
      ::Draw1Control(cMacControl);
    } else {
      if (buttonBitmap) {
	int btop;
	btop = (cWindowHeight - buttonBitmap->GetHeight()) / 2;
	buttonBitmap->DrawMac(IR_CIRCLE_SIZE + IR_X_SPACE, btop);
      } else if (labelString) {
	Rect r = { SetOriginY, IR_CIRCLE_SIZE + IR_X_SPACE + SetOriginX, 
		   SetOriginY + cWindowHeight, SetOriginX + cWindowWidth };
	CFStringRef str;

	str = CFStringCreateWithCString(NULL, labelString, kCFStringEncodingISOLatin1);
	
	DrawThemeTextBox(str, kThemeSystemFont, kThemeStateActive,
			 0, &r, teJustLeft, NULL);

	CFRelease(str);
      }

      {
	int top = (cWindowHeight - IR_CIRCLE_SIZE) / 2;
	Rect r = { top, 0, top + IR_CIRCLE_SIZE, IR_CIRCLE_SIZE };
	OffsetRect(&r,SetOriginX,SetOriginY);
	PenSize(1, 1);
	ForeColor(blackColor);
	FrameOval(&r);
	InsetRect(&r, 1, 1);
	ForeColor(whiteColor);
	PaintOval(&r);
	ForeColor(blackColor);
	if (bitmapState) {
	  InsetRect(&r, IR_ON_INSET - 1, IR_ON_INSET - 1);
	  PaintOval(&r);
	}
	cMacDC->setCurrentUser(NULL);
      }
    }
  }
}

void wxRadioButton::Highlight(Bool on)
{
  int top = (cWindowHeight - IR_CIRCLE_SIZE) / 2;
  Rect r = { top + 1, 1, top + IR_CIRCLE_SIZE - 1, IR_CIRCLE_SIZE - 1};

  OffsetRect(&r,SetOriginX,SetOriginY);
  if (!on)
    ForeColor(whiteColor);
  PenSize(1, 1);
  FrameOval(&r);
  if (!on) ForeColor(blackColor);
}

//-----------------------------------------------------------------------------
void wxRadioButton::DoShow(Bool show)
{
  if (!CanShow(show)) return;

  if (cMacControl) {
    if (show) {
      ::ShowControl(cMacControl);
    } else {
      ::HideControl(cMacControl);
    }
  }

  wxWindow::DoShow(show);
}

//-----------------------------------------------------------------------------
void wxRadioButton::OnEvent(wxMouseEvent *event) // mac platform only
{
  if (cEnable) {
    if (event->LeftDown()) {
      int startH, startV;
      Point startPt;
      int trackResult;

      SetCurrentDC();
      
      event->Position(&startH, &startV); // client c.s.
      
      startPt.v = startV + SetOriginY; // port c.s.
      startPt.h = startH + SetOriginX;
      if (::StillDown()) {
	if (cMacControl)
	  trackResult = ::TrackControl(cMacControl, startPt, NULL);
	else
	  trackResult = Track(startPt);
      } else
	trackResult = 1;
      if (trackResult) {
	wxCommandEvent *commandEvent;
	commandEvent = new wxCommandEvent(wxEVENT_TYPE_RADIOBOX_COMMAND);
	ProcessCommand(commandEvent);
      }
    }
  }
}

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Sizing methods
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

//-----------------------------------------------------------------------------
void wxRadioButton::OnClientAreaDSize(int dW, int dH, int dX, int dY) // mac platform only
{
  int clientWidth, clientHeight;

  if (!cMacControl) return;

  SetCurrentDC();

  GetClientSize(&clientWidth, &clientHeight);

  ::SizeControl(cMacControl, clientWidth, clientHeight);
  ::MoveControl(cMacControl, SetOriginX, SetOriginY);
}
